from __future__ import unicode_literals

from six import with_metaclass

from manager import Manager
from options import Options

class ModelBase(type):
	def __new__(cls, name, bases, attrs):
		"""
		Also ensure initialization is only performed for subclasses of Model,
        excluding Model class itself).
		"""
		parents = [b for b in bases if isinstance(b, ModelBase)]
		if not parents:
			return super(ModelBase, cls).__new__(cls, name, bases, attrs)

		module = attrs.pop('__module__')
		attr_meta = attrs.pop('Meta', None)
		
		new_class = type.__new__(cls, name, bases, {'__module__': module})
		
		if not attr_meta:
			meta = getattr(new_class, 'Meta', None)
		else:
			meta = attr_meta
        
		new_class.add_to_class('_meta', Options(meta))	
		new_class.add_to_class('objects', Manager())

		return new_class

	def add_to_class(cls, name, value):
		if hasattr(value, 'contribute_to_class'):
			value.contribute_to_class(cls, name)
		else:
			setattr(cls, name, value)
		
class Node(with_metaclass(ModelBase)):
	pass


import inspect
import six

from query import QuerySet

class BaseManager(object):
	
	def __init__(self):
		super(BaseManager, self).__init__()
		self.node = None

	def __str__(self):
		return '<Neo4j Node: {0}>'.format(self.node)

	def contribute_to_class(self, node, name):
		self.node = node
		setattr(node, name, self)

	@classmethod
	def _get_queryset_methods(cls, queryset_class):
		def create_method(name, method):
			def manager_method(self, *args, **kwargs):
				return getattr(self.get_queryset(), name)(*args, **kwargs)
			manager_method.__name__ = method.__name__
			manager_method.__doc__ = method.__doc__
			return manager_method
		
		new_methods = {}
		
		predicate = inspect.isfunction if six.PY3 else inspect.ismethod
		for name, method in inspect.getmembers(queryset_class, predicate=predicate):
			
			if hasattr(cls, name):
				continue
			
			queryset_only = getattr(method, 'queryset_only', None)
			if queryset_only or (queryset_only is None and name.startswith('_')):
				continue
			
			new_methods[name] = create_method(name, method)

		return new_methods

	def get_queryset(self):
		return self._queryset_class(cls_=self.node)
        
	@classmethod
	def from_queryset(cls, queryset_class, class_name=None):
		if class_name is None:
			class_name = '%sFrom%s' % (cls.__name__, queryset_class.__name__)
		class_dict = {
			'_queryset_class': queryset_class,
		}
		class_dict.update(cls._get_queryset_methods(queryset_class))
		return type(class_name, (cls,), class_dict)

class Manager(BaseManager.from_queryset(QuerySet)):
	pass


from __future__ import unicode_literals

from django.conf import settings as _settings

def URLConnection():
	try:
		if not 'NEO4J_DATABASES' in dir(_settings):
			raise Exception("Not have 'NEO4J_DATABASES' in settings.py. Read more: http://goo.gl/TnbmHo/")
		
		if not type(_settings.NEO4J_DATABASES) is dict:
			raise Exception("NEO4J_DATABASES is not configured correctly in settings.py. Read more: http://goo.gl/TnbmHo/")

		if not 'default' in _settings.NEO4J_DATABASES.keys():
			raise Exception("NEO4J_DATABASES is not configured correctly in settings.py. Read more: http://goo.gl/TnbmHo/")
		
		_default = _settings.NEO4J_DATABASES['default']
		
		if not 'HOST' in _default \
			and not 'PORT' in _default \
			and not 'ENDPOINT' in _default:
			raise Exception("NEO4J_DATABASES is not configured correctly in settings.py. Read more: http://goo.gl/TnbmHo/")

		protocol = "http"
		host = _default['HOST']
		port = _default['PORT']
		endpoint = _default['ENDPOINT']

		_url = "%s://%s:%s%s" % (protocol, host, port, endpoint)
		return _url
	except Exception, e:
		raise e


# -*- coding: utf-8 -*-

from neodjango.core.connection import Connection

class NodeRelationshipQuerySet(object):

	def __init__(self, relationship, *args, **kwargs):
		self.relationship = relationship

	def __repr__(self):
		_id_relationship = self.relationship.id
		_display = self.relationship.url
		
		return '<Relationship#{0}: {1}>'.format(_id_relationship, _display)

	def __getattr__(self, name, *args, **kwargs):
		if name in self.relationship.end.properties:
			return self.relationship.end.properties[name]
		raise KeyError("Relationship#{0} has no property {1}. Read more: http://goo.gl/TnbmHo/".format(self.relationship.id, name))

	@classmethod
	def _create_relationship(self):
		pass

class NodeQuerySet(object):
	PROPERTY = None
	RELATIONSHIP_TEXT = lambda self, text: text.upper()

	def __init__(self, node, *args, **kwargs):
		self.node = node

	def __getattr__(self, name, *args, **kwargs):
		if name == 'relationships':
			name = self.RELATIONSHIP_TEXT(name)
			def wrapper(*args, **kwargs):
				if len(args)>=0:
					return None
				return RelationshipQuerySet()
			return wrapper
		else:
			return self.node.properties[name]

	def __len__(self):
		return 1

	def __repr__(self):
		_id_node = self.node.id
		_display = self.node.url
		
		if not self.PROPERTY is None:
			_display = self.node.properties[self.PROPERTY].encode('ascii', 'ignore')
		
		return '<Node#{0}: {1}>'.format(_id_node, _display)

	def relationships(self):
		relationships = self.node.relationships.all()
		return [NodeRelationshipQuerySet(relationship) for relationship in relationships]


class QuerySet(object):
	def __init__(self, cls_, *args, **kwargs):
		self.graph = Connection()
		self.cls_ = cls_
		if not self.cls_._meta['label'] is None:
			self._nodes_by_labels = self.graph.labels.get(self.cls_._meta['label'])
			self.label = self.cls_._meta['label']
		else:
			self._nodes_by_labels = self.graph.labels.get(self.cls_.__name__)
			self.label = self.cls_.__name__

		if not self.cls_._meta['display'] is None:
			NodeQuerySet.PROPERTY = self.cls_._meta['display']
		
	def _node_filter(self, **kwargs):
		result = [NodeQuerySet(node) for node in self._nodes_by_labels.get(**kwargs)]
		result.reverse()
		return result

	def _node_get(self, id_node):
		result = self.graph.nodes.get(id_node)
		return NodeQuerySet(result)

	def _node_all(self):
		return self._node_filter()

	def filter(self, **kwargs):
		return self._node_filter(**kwargs)

	def get(self, **kwargs):
		if 'id' in kwargs.keys():
			result = self._node_get(kwargs['id'])
		else:
			result = self._node_filter(**kwargs)
			result = result[0]

		if len(result)!=1:
			raise ValueError("None has no property. Read more: http://goo.gl/TnbmHo/")
		return result
	
	def all(self):
		return self._node_all()


import os

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

NEO4J_DATABASES = {
    'default' : {
        'HOST':'localhost',
        'PORT': 7474,
        'ENDPOINT': '/db/data',   
    },
}

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db', 'test_database.sqlite3')
    }
}

INSTALLED_APPS = (
    'neodjango', 
)

SECRET_KEY="shutupdjangowe'retryingtotesthere"

DEBUG = True


from setuptools import setup, find_packages

EXCLUDE_FROM_PACKAGES = ['neodjango.test', ]

setup(
    name = 'neodjango',
    version = '0.0.7',
    author ='Renan Palmeira',
    author_email ='renanpalmeira1@hotmail.com',
    description ='A Django/Neo4j ORM layer.',
    license = 'GPL',
    url = 'https://neodjango.readthedocs.org/en/latest/',
    keywords = 'django neo4j graph graphdb graphdatabase database rest client driver',
    packages = find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    include_package_data = True,
    long_description = open('README.md').read(),
    install_requires = [
        'neo4jrestclient>=2.1',
        'Django',
        'requests',
        'six',
        'wsgiref',
    ],
    tests_require = [
        'nose>=1.0',
        'requests>=0.4.1',
        'nose-regression>=1.0',
    ],
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
    ],
)


from __future__ import unicode_literals

class Options(object):
	def __init__(self, meta, *args, **kwargs):
	    self.meta = meta

	def __getitem__(self, value):
		if hasattr(self.meta, value):
			return getattr(self.meta, value, 'url')
		return None